# Copyright 2025 Wolfskämpf
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import dataclasses
import decimal
import enum

import htpy as h
from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import HTMLResponse
from starlette.routing import Mount, Route
from starlette.staticfiles import StaticFiles


class Unit(enum.StrEnum):
    mg = "mg"
    g = "g"


@dataclasses.dataclass()
class ActiveIngredient:
    name: str
    amount: decimal.Decimal
    unit: Unit


@dataclasses.dataclass()
class Medication:
    name: str
    active_ingredient: ActiveIngredient
    manufacturer: int  # pretend this is a ForeignKey


medication_table = [
    Medication(
        name="Estreva",
        active_ingredient=ActiveIngredient(
            "Estradiol", amount=decimal.Decimal("5"), unit=Unit.mg
        ),
        manufacturer=13,
    ),
    Medication(
        name="Utrogest",
        active_ingredient=ActiveIngredient(
            name="Progesterone",
            amount=decimal.Decimal("200"),
            unit=Unit.mg,
        ),
        manufacturer=12,
    ),
]


def medication_tr(medication: Medication) -> h.Element:
    return h.tr[
        h.td[medication.name],
        h.td[medication.active_ingredient.name],
        h.td[
            f"{medication.active_ingredient.amount} {medication.active_ingredient.unit}"
        ],
        h.td[h.input(type="checkbox")],
    ]


def medication_plan(medications: list[Medication]) -> h.Element:
    return h.table[
        h.thead[
            h.tr[
                h.th(scope="column")["Name"],
                h.th(scope="column")["Active Ingredient"],
                h.th(scope="column")["Amount"],
                h.th(scope="column")["Taken"],
            ]
        ],
        h.tbody[(medication_tr(medication) for medication in medications)],
    ]


def base(main_content: h.Node) -> h.Element:
    return h.html(lang="en")[
        h.head[
            h.meta(charset="utf-8"),
            h.meta(name="viewport", content="width=device-width, initial-scale=1"),
            h.meta(name="color-scheme", content="light dark"),
            h.link(rel="stylesheet", href="static/css/pico.fuchsia.min.css"),
            h.title["Medication Planner"],
        ],
        h.body[h.main(".container")[main_content]],
    ]


async def index(request: Request) -> HTMLResponse:
    return HTMLResponse(
        base(
            main_content=[
                h.h1["Medication Planner"],
                medication_plan(medication_table),
                h.p[
                    "This is an example, not medical advice. Trans Rights are Human Rights 🏳️‍⚧️"
                ],
            ]
        )
    )


app = Starlette(
    routes=[
        Route("/", index),
        Mount("/static", app=StaticFiles(directory="static"), name="static"),
    ]
)
